#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from scipy import *
from scipy.special import *
from scipy.integrate import *
from pylab import *
from openacoustics.gmsh import *
from openacoustics.bem import *

# Gmsh geometry file name
geo = 'square.geo'

# Create an instance of class gmsh
gmsh = GMsh2D()
gmsh.loadGeo(geo, 1)
#gmsh.viewGeo(geo)

nodes = gmsh.getNodes()
sf = gmsh.getLines("")

nodenum = nodes.shape[0]
sfnum = sf.shape[0]
M = sfnum

S = zeros(M, 'float64')
r = zeros((M, 2), 'float64')
nv = zeros((M, 2), 'float64')

for ii in range(M):
  node1 = nodes[sf[ii, 0], :]
  node2 = nodes[sf[ii, 1], :]

  # Element surface area
  S[ii] = linalg.norm(node2 - node1)

  # Centroid of the element
  r[ii, :] = (node1 + node2) / 2

  # Surface normal of the element
  v1 = node2 - node1
  normal = array([v1[1], -v1[0]])
  nv[ii, :] = normal / linalg.norm(normal)

figure
hold("on")
for ii in range(sfnum):
    node1 = nodes[sf[ii, 0], :]
    node2 = nodes[sf[ii, 1], :]
    mp = r[ii, :]
    plot(array([node1[0], node2[0]]), array([node1[1], node2[1]]))
    plot(array([mp[0], mp[0] + 0.05 * nv[ii,0]]),
         array([mp[1], mp[1] + 0.05 * nv[ii,1]]))
    #quiver(array([mp[0], mp[0] + nv[ii, 0]]),
    #       array([mp[1], mp[1] + nv[ii, 1]]))
hold("off")

beta = 0
w = 2 * pi * 170
c = 343
ro0 = 1.21
k = w / c

bem = Bem2D()
A = bem.createMatrix(r,nv,S,k,w,ro0,beta)

# Source location
rs = [0., 0.]
Pd = bem.createSourceMatrix(rs,r,k)


I = eye(A.shape[0])
Phi = linalg.solve(0.5 * I - A, Pd)

dx = 0.1 / 2
dy = 0.1 / 2

xx = arange(dx, 1, dx)
yy = arange(dy, 1, dy)

Pr = bem.calcGrid(Phi,xx,yy,rs,r,nv,S,k,w,ro0,beta)

X, Y = meshgrid(xx, yy)

figure()
contourf(X, Y, 20 * log10(abs(Pr)))
show()
