#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import numpy as np

## dB to energy
#  Transform data in dB unit to energy order.
#  @param dB data
#  @retval energy order data
def dB2ene(dB):
    """dB to energy"""

    arr = np.asarray(dB)

    ene = 10. ** (arr / 10.)

    return(ene)

## energy to dB
#  Transform data in energy order to dB unit data.
#  @param energy order data
#  @retval data in dB unit
def ene2dB(ene):
    """energy to dB"""

    dB = 10 * np.log10(ene)

    return(dB)

## Take energetically summation of dB data
#  @param data in dB unit
#  @param axis over whitch the sum is taken, defalut=0
#  @retval result of snergetically summation
def esum(dB, ax = 0):
    """
    ret = esum( dB, dim = 0 )

    dB : dB data
    ax : axis over which the sum is taken
    ret : result of enegetically summation

    Take enegetically summation of dB data.
    """

    ene = dB2ene(dB)
    ene = np.sum(ene, ax)
    ret = ene2dB(ene)
     
    return(ret)

def emean(dB, ax = 0):
    """
    ret = emean( dB, dim = 0 )

    dB : dB data
    ax : axis over which the mean is taken
    ret : result of enegetically mean

    Take enegetically mean of dB data.
    """

    ene = dB2ene(dB)
    ene = np.mean(ene, ax)
    ret = ene2dB(ene)

    return(ret)


def octave(freq, data, div):
    
    nfreq, ndata = data.shape

    #if nfreq != size(freq):

    #== center and cutoff frequencies of bands ==
    cf = 10. ** np.arange(-0.3, 5.2, 0.3 / div)
    lf = cf * 10. ** (-0.3 / (2. * div))
    uf = cf * 10. ** ( 0.3 / (2. * div)) 

    #== find bands including freq ==
    ind = np.nonzero( (freq[0] < uf) * (lf <= freq[-1]) )[0]

    #== process routine ==
    oct = np.zeros((len(ind), ndata))
    ene = dB2ene(data)
    k = 0
    for n in ind:
       tmp = (lf[n] <= freq) * (freq < uf[n])
       oct[k,:] = 10. * np.log10( np.mean( ene[tmp,:] ) )
       k = k + 1

    return cf[ind], oct

    








