#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from openacoustics.gmsh import *
from openacoustics.fem import *

order=2

# Gmsh geometry file name
geo = 'ahlv100rGmsh/ahlv100r.geo'
msh = 'ahlv100rGmsh/ahlv100r.msh'

# Create an instance of class gmsh
gmsh = GMsh2D()

# Create an instance of class Fem
fem = Fem2D()

# Display the .geo file
gmsh.viewGeo(geo)

# Load the .geo file
gmsh.loadMsh(msh)
#gmsh.loadGeo(geo,1,order)


# Load node indices and coordinates
nodes = gmsh.getNodes()
nodenum = nodes.shape[0] # Number of nodes


# Load node numbers of triangular elements
elms = gmsh.getTriangles("", order)
elmnum = elms.shape[0] # The number of triangular elements

source = gmsh.getLines(5,order)
absorb = gmsh.getLines(6,order)

#gmsh.plotTriangularElements(elms,nodes[:,0],nodes[:,1])

# Prepare global matrices for K and M
K = sparse.lil_matrix((nodenum, nodenum),dtype='complex')
M = sparse.lil_matrix((nodenum, nodenum),dtype='complex')
C = sparse.lil_matrix((nodenum, nodenum),dtype='complex')

# Start creating the global matrices
# Append the triangular elements into the global matrices
for N in range(elmnum):
    # Load the nodes that consists the Nth triangular element
    node1 = nodes[elms[N, 0], :]
    node2 = nodes[elms[N, 1], :]
    node3 = nodes[elms[N, 2], :]

    # Calculate the element matrices for the triangular element
    Kn, Mn, Bn, S = fem.triangularElement(node1, node2, node3, order)

    # Divide the mass matrix by the square of the sound speed
    # assuming air medium
    Mn = Mn / (343. * 343.)

    idx = elms[N, :]
    for ii in range(Kn.shape[0]):
        for jj in range(Kn.shape[1]):
            K[idx[ii], idx[jj]] += Kn[ii, jj]
            M[idx[ii], idx[jj]] += Mn[ii, jj]

if order==1:
    r=0
    for N in range(absorb.shape[0]):
        node1 = nodes[absorb[N,0],:]
        node2 = nodes[absorb[N,1],:]

        Cn = array([[2., 1.],
                    [1., 2.]]) *  sqrt(sum((node2-node1)**2)) / 6.

        idx = absorb[N,:]
        for ii in range(Cn.shape[0]):
            for jj in range(Cn.shape[1]):
                C[ idx[ii], idx[jj] ] -= 1.3*Cn[ ii,jj ]/445.9

if order==2:
    r=0
    for N in range(absorb.shape[0]):
        node1 = nodes[absorb[N,0],:]
        node2 = nodes[absorb[N,1],:]

        Cn = array([[4., -1., 2.],
                    [-1., 4.,  2.],
                    [2., 2.,  16.]]) *  sqrt(sum((node2-node1)**2)) / 30.

        idx = absorb[N,:]
        for ii in range(Cn.shape[0]):
            for jj in range(Cn.shape[1]):
                C[ idx[ii], idx[jj] ] -= 1.3*Cn[ ii,jj ]/445.9

# Frequency
f = 500

w = 2. * pi * f
q = zeros(nodenum,dtype='complex')

if order==1:
    for N in range(source.shape[0]):
        node1 = nodes[source[N,0],:]
        node2 = nodes[source[N,1],:]

        q[ source[N,0] ] += sqrt(sum((node2-node1)**2)) * 0.014 / 2.
        q[ source[N,1] ] += sqrt(sum((node2-node1)**2)) * 0.014 / 2.

if order==2:
    for N in range(source.shape[0]):
        node1 = nodes[source[N,0],:]
        node2 = nodes[source[N,1],:]

        q[ source[N,0] ] += sqrt(sum((node2-node1)**2)) * 0.014 / 6.
        q[ source[N,1] ] += sqrt(sum((node2-node1)**2)) * 0.014 / 6.
        q[ source[N,2] ] += sqrt(sum((node2-node1)**2)) * 0.014 * 4. / 6.

# Unit divergence at the source location

# Solve the global matrix system
phi = linsolve.spsolve(K - (w**2) * M + 1j*w*C, 1j*w*1.3*q.T)

# Extract x and y coordinates
x = nodes[:, 0]
y = nodes[:, 1]

# Calculate the dB values
z = 20 * log10(abs(phi))

# The linear absolute value
z = real(phi)
#
# Display spatial distribution of velocity potential with contour plot
gmsh.contour(elms, x, y, z, "Relative sound pressure [dB]", "x[m]", "y[m]", order)

A = phi[2-1]
B = phi[67-1]
C = phi[1-1]
D = phi[64-1]

print "A=",A
print "B=",B
print "C=",C
print "D=",D

A = phi[1-1]
B = phi[63-1]
C = phi[2-1]
D = phi[66-1]

print "A=",A
print "B=",B
print "C=",C
print "D=",D
