#  ooo  oooo  oooo o   o     o    ooo   ooo  o   o  oooo ooooo o  ooo   oooo
# 0   0 0   0 0    00  0    0 0  0   0 0   0 0   0 0       0   0 0   0 0
# 0   0 oooo  oooo 0 0 0   0   0 0     0   0 0   0  oooo   0   0 0      oooo
# 0   0 o     0    0 o 0   ooooo 0   o 0   0 0   0      0  0   0 0   o      0
#  ooo  o     oooo o  0o  0     0 ooo   ooo   ooo  0oooo   0   0  ooo  0oooo
#
#  (  (  (  (  (  (  (  (  (  (  (  (  O  )  )  )  )  )  )  )  )  )  )  )  )
#
#    OpenAcoustics: The Open Source Collection for Computational Acoustics
#
#               Copyright (C) 2008-2009 The OpenAcoustics Team
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from scipy import *
from openacoustics.gmsh import *
from openacoustics.fem import *

import time
t1=time.time()

order=2

# Gmsh geometry file name
#geo = 'ahlv100rGmsh/ahlv100r.geo'

if order==1:
    msh = 'ahlv100kGmsh/ahlv100k.msh'

if order==2:
    msh = 'ahlv100jGmsh/ahlv100j.msh'

# Create an instance of class gmsh
gmsh = GMsh3D()

# Create an instance of class Fem
fem = Fem3D()

# Display the .geo file
#gmsh.viewGeo(geo)

# Load the .geo file
gmsh.loadMsh(msh)
#gmsh.loadGeo(geo,1,order)


# Load node indices and coordinates
nodes = gmsh.getNodes()
nodenum = nodes.shape[0] # Number of nodes


# Load node numbers of triangular elements
elms = gmsh.getTetrahedra("", order)
elmnum = elms.shape[0] # The number of triangular elements

#triangles = gmsh.getTriangles("",order)
#
#source = triangles[0:8,:]
#absorb = triangles[8:,:]

source = gmsh.getTriangles(2,order)
absorb = gmsh.getTriangles(3,order)


# Prepare global matrices for K and M
K = scipy.sparse.lil_matrix((nodenum, nodenum),dtype='complex')
M = scipy.sparse.lil_matrix((nodenum, nodenum),dtype='complex')
C = scipy.sparse.lil_matrix((nodenum, nodenum),dtype='complex')

# Start creating the global matrices
# Append the triangular elements into the global matrices
for N in range(elmnum):
    # Load the nodes that consists the Nth triangular element
    node1 = nodes[elms[N, 0], :]
    node2 = nodes[elms[N, 1], :]
    node3 = nodes[elms[N, 2], :]
    node4 = nodes[elms[N, 3], :]

    # Calculate the element matrices for the triangular element
    Kn, Mn, V = fem.tetrahedralElement(node1, node2, node3, node4, order)

    # Divide the mass matrix by the square of the sound speed
    # assuming air medium
    Mn = Mn / (343. * 343.)

    idx = elms[N, :]
    for ii in range(Kn.shape[0]):
        for jj in range(Kn.shape[1]):
            K[idx[ii], idx[jj]] += Kn[ii, jj]
            M[idx[ii], idx[jj]] += Mn[ii, jj]

for N in range(absorb.shape[0]):
    node1 = nodes[absorb[N,0],:]
    node2 = nodes[absorb[N,1],:]
    node3 = nodes[absorb[N,2],:]
    
    Kn,Cn,S=fem.triangularElement(node1, node2, node3, order)

    idx = absorb[N,:]
    for ii in range(Cn.shape[0]):
        for jj in range(Cn.shape[1]):
            C[ idx[ii], idx[jj] ] -= 1.3*Cn[ ii,jj ]/445.9

# Frequency
f = 500

w = 2. * pi * f
q = zeros(nodenum,dtype='complex')

if order==1:
    for N in range(source.shape[0]):
        node1 = nodes[source[N,0],:]
        node2 = nodes[source[N,1],:]
        node3 = nodes[source[N,2],:]
        
        a = node2 - node1
        b = node3 - node2
        c = node1 - node3

        s = (linalg.norm(a) + linalg.norm(b) + linalg.norm(c)) / 2
        S = sqrt(s * (s - linalg.norm(a)) * (s - linalg.norm(b))
                 * (s - linalg.norm(c)))

        q[ source[N,0] ] += S * 0.014 / 3.
        q[ source[N,1] ] += S * 0.014 / 3.
        q[ source[N,2] ] += S * 0.014 / 3.

if order==2:
    for N in range(source.shape[0]):
        node1 = nodes[source[N,0],:]
        node2 = nodes[source[N,1],:]
        node3 = nodes[source[N,2],:]
        
        a = node2 - node1
        b = node3 - node2
        c = node1 - node3

        s = (linalg.norm(a) + linalg.norm(b) + linalg.norm(c)) / 2
        S = sqrt(s * (s - linalg.norm(a)) * (s - linalg.norm(b))
                 * (s - linalg.norm(c)))

#        q[ source[N,0] ] += S * 0.014 / 3.
#        q[ source[N,1] ] += S * 0.014 / 3.
#        q[ source[N,2] ] += S * 0.014 / 3.

        q[ source[N,3] ] += S * 0.014 / 3.
        q[ source[N,4] ] += S * 0.014 / 3.
        q[ source[N,5] ] += S * 0.014 / 3.

        
#        q[ source[N,0] ] += S * 0.014 / 9.
#        q[ source[N,1] ] += S * 0.014 / 9.
#        q[ source[N,2] ] += S * 0.014 / 9.
#        q[ source[N,3] ] += S * 2 * 0.014 / 9.
#        q[ source[N,4] ] += S * 2 * 0.014 / 9.
#        q[ source[N,5] ] += S * 2 * 0.014 / 9.
#
#        q[ source[N,0] ] += S * 0.014 / 6.
#        q[ source[N,1] ] += S * 0.014 / 6.
#        q[ source[N,2] ] += S * 0.014 / 6.
#        q[ source[N,3] ] += S * 0.014 / 6.
#        q[ source[N,4] ] += S * 0.014 / 6.
#        q[ source[N,5] ] += S * 0.014 / 6.

#        q[ source[N,0] ] += S * 0.014 / 15.
#        q[ source[N,1] ] += S * 0.014 / 15.
#        q[ source[N,2] ] += S * 0.014 / 15.
#        q[ source[N,3] ] += S * 4 * 0.014 / 15.
#        q[ source[N,4] ] += S * 4 * 0.014 / 15.
#        q[ source[N,5] ] += S * 4 * 0.014 / 15.
#
# Solve the global matrix system
phi = scipy.sparse.linalg.spsolve(K - (w**2) * M + 1j*w*C, 1j*w*1.3*q.T)

t2=time.time()

print t2-t1

# Extract x and y coordinates
x = nodes[:, 0]
y = nodes[:, 1]
z = nodes[:, 2]

# The linear absolute value
phi = real(phi)

#Display spatial distribution of velocity potential with contour plot
#gmsh.contour(elms, x, y, z, "Relative sound pressure [dB]", "x[m]", "y[m]", order)

scalars=phi
gmsh.beginPostprocessing()
gmsh.beginView("Slice of sound field")

if order==1:
    gmsh.addTetrahedron(elms, x, y, z, scalars)
elif order==2:
    gmsh.addTetrahedron2nd(elms, x, y, z, scalars)
gmsh.closeView()
#gmsh.addCutPlaneByABCDForm(-1, 1, 1, 1, -1)
#gmsh.title(1, "Slice of potential velocity field")
#gmsh.xLabel(1, "x[m]")
#gmsh.yLabel(1, "y[m]")
#gmsh.zLabel(1, "z[m]")
#gmsh.addStyle()
#gmsh.merge(geo)
gmsh.endPostprocessing()
gmsh.run()


if order==1:
    A = phi[6-1]
    B = phi[37-1]
    C = phi[3-1]
    D = phi[46-1]

    print "A=",A
    print "B=",B
    print "C=",C
    print "D=",D

if order==2:
    A = phi[4-1]
    B = phi[18-1]
    C = phi[7-1]
    D = phi[17-1]

    print "A=",A
    print "B=",B
    print "C=",C
    print "D=",D
